@@ -1,5 +1,7 @@
 """Test for tmuxp workspace builder."""
 
+from __future__ import annotations
+
 import functools
 import os
 import pathlib
@@ -13,21 +15,20 @@ from libtmux._internal.query_list import ObjectDoesNotExist
 from libtmux.common import has_gte_version, has_lt_version
 from libtmux.exc import LibTmuxException
 from libtmux.pane import Pane
-from libtmux.server import Server
 from libtmux.session import Session
 from libtmux.test import retry_until, temp_session
 from libtmux.window import Window
 
+from tests.constants import EXAMPLE_PATH, FIXTURE_PATH
+from tests.fixtures import utils as test_utils
 from tmuxp import exc
 from tmuxp._internal.config_reader import ConfigReader
 from tmuxp.cli.load import load_plugins
 from tmuxp.workspace import loader
 from tmuxp.workspace.builder import WorkspaceBuilder
 
-from ..constants import EXAMPLE_PATH, FIXTURE_PATH
-from ..fixtures import utils as test_utils
-
 if t.TYPE_CHECKING:
+    from libtmux.server import Server
 
     class AssertCallbackProtocol(t.Protocol):
         """Assertion callback type protocol."""
@@ -95,12 +96,12 @@ def test_focus_pane_index(session: Session) -> None:
 
     assert session.active_window.name == "focused window"
 
-    _pane_base_index = session.active_window.show_window_option(
+    pane_base_index_ = session.active_window.show_window_option(
         "pane-base-index",
         g=True,
     )
-    assert isinstance(_pane_base_index, int)
-    pane_base_index = int(_pane_base_index)
+    assert isinstance(pane_base_index_, int)
+    pane_base_index = int(pane_base_index_)
 
     pane_base_index = 0 if not pane_base_index else int(pane_base_index)
 
@@ -111,7 +112,7 @@ def test_focus_pane_index(session: Session) -> None:
         if pane is not None and pane.index is not None
     ]
 
-    pane_indexes_should_be = [pane_base_index + x for x in range(0, 3)]
+    pane_indexes_should_be = [pane_base_index + x for x in range(3)]
     assert pane_indexes_should_be == pane_base_indexes
 
     w = session.active_window
@@ -216,9 +217,9 @@ def test_suppress_history(session: Session) -> None:
 
             return assertCase(sent_cmd, history_cmd)
 
-        _f = functools.partial(f, p=p, buffer_name=buffer_name, assertCase=assertCase)
+        f_ = functools.partial(f, p=p, buffer_name=buffer_name, assertCase=assertCase)
 
-        assert retry_until(_f), f"Unknown sent command: [{sent_cmd}] in {assertCase}"
+        assert retry_until(f_), f"Unknown sent command: [{sent_cmd}] in {assertCase}"
 
 
 def test_session_options(session: Session) -> None:
@@ -231,13 +232,13 @@ def test_session_options(session: Session) -> None:
     builder = WorkspaceBuilder(session_config=workspace, server=session.server)
     builder.build(session=session)
 
-    _default_shell = session.show_option("default-shell")
-    assert isinstance(_default_shell, str)
-    assert "/bin/sh" in _default_shell
+    default_shell = session.show_option("default-shell")
+    assert isinstance(default_shell, str)
+    assert "/bin/sh" in default_shell
 
-    _default_command = session.show_option("default-command")
-    assert isinstance(_default_command, str)
-    assert "/bin/sh" in _default_command
+    default_command = session.show_option("default-command")
+    assert isinstance(default_command, str)
+    assert "/bin/sh" in default_command
 
 
 def test_global_options(session: Session) -> None:
@@ -250,10 +251,10 @@ def test_global_options(session: Session) -> None:
     builder = WorkspaceBuilder(session_config=workspace, server=session.server)
     builder.build(session=session)
 
-    _status_position = session.show_option("status-position", _global=True)
-    assert isinstance(_status_position, str)
-    assert "top" in _status_position
-    assert session.show_option("repeat-time", _global=True) == 493
+    status_position = session.show_option("status-position", global_=True)
+    assert isinstance(status_position, str)
+    assert "top" in status_position
+    assert session.show_option("repeat-time", global_=True) == 493
 
 
 def test_global_session_env_options(
@@ -276,9 +277,9 @@ def test_global_session_env_options(
     builder = WorkspaceBuilder(session_config=workspace, server=session.server)
     builder.build(session=session)
 
-    _visual_silence = session.show_option("visual-silence", _global=True)
-    assert isinstance(_visual_silence, str)
-    assert visual_silence in _visual_silence
+    visual_silence_ = session.show_option("visual-silence", global_=True)
+    assert isinstance(visual_silence_, str)
+    assert visual_silence in visual_silence_
     assert repeat_time == session.show_option("repeat-time")
     assert main_pane_height == session.active_window.show_window_option(
         "main-pane-height",
@@ -377,9 +378,9 @@ def test_window_shell(
         def f(w: Window) -> bool:
             return w.window_name != "top"
 
-        _f = functools.partial(f, w=w)
+        f_ = functools.partial(f, w=w)
 
-        retry_until(_f)
+        retry_until(f_)
 
         assert w.name != "top"
 
@@ -482,11 +483,13 @@ def test_environment_variables_warns_prior_to_tmux_3_0(
 
 
 def test_automatic_rename_option(
-    server: "Server",
+    server: Server,
     monkeypatch: pytest.MonkeyPatch,
 ) -> None:
     """Test workspace builder with automatic renaming enabled."""
     monkeypatch.setenv("DISABLE_AUTO_TITLE", "true")
+    monkeypatch.setenv("ROWS", "36")
+
     workspace = ConfigReader._from_file(
         test_utils.get_workspace_file("workspace/builder/window_automatic_rename.yaml"),
     )
@@ -513,10 +516,10 @@ def test_automatic_rename_option(
 
     def check_window_name_match() -> bool:
         assert w.show_window_option("automatic-rename") == "on"
-        return (
-            w.name == pathlib.Path(os.getenv("SHELL", "bash")).name
-            or w.name == portable_command
-        )
+        return w.name in {
+            pathlib.Path(os.getenv("SHELL", "bash")).name,
+            portable_command,
+        }
 
     assert retry_until(
         check_window_name_match,
@@ -591,10 +594,10 @@ def test_start_directory(session: Session, tmp_path: pathlib.Path) -> None:
                     pane_path is not None and path in pane_path
                 ) or pane_path == path
 
-            _f = functools.partial(f, path=path, p=p)
+            f_ = functools.partial(f, path=path, p=p)
 
             # handle case with OS X adding /private/ to /tmp/ paths
-            assert retry_until(_f)
+            assert retry_until(f_)
 
 
 def test_start_directory_relative(session: Session, tmp_path: pathlib.Path) -> None:
@@ -646,10 +649,10 @@ def test_start_directory_relative(session: Session, tmp_path: pathlib.Path) -> N
                     pane_path is not None and path in pane_path
                 ) or pane_path == path
 
-            _f = functools.partial(f, path=path, p=p)
+            f_ = functools.partial(f, path=path, p=p)
 
             # handle case with OS X adding /private/ to /tmp/ paths
-            assert retry_until(_f)
+            assert retry_until(f_)
 
 
 @pytest.mark.skipif(
@@ -725,9 +728,9 @@ def test_pane_order(session: Session) -> None:
                 p.refresh()
                 return p.pane_current_path == pane_path
 
-            _f = functools.partial(f, pane_path=pane_path, p=p)
+            f_ = functools.partial(f, pane_path=pane_path, p=p)
 
-            assert retry_until(_f)
+            assert retry_until(f_)
 
 
 def test_window_index(
@@ -1096,25 +1099,34 @@ def test_find_current_active_pane(
     assert builder.find_current_attached_session() == second_session
 
 
-@pytest.mark.parametrize(
-    "yaml,output,should_see",
-    [
-        [
-            textwrap.dedent(
-                """
+class WorkspaceEnterFixture(t.NamedTuple):
+    """Test fixture for workspace enter behavior verification."""
+
+    test_id: str
+    yaml: str
+    output: str
+    should_see: bool
+
+
+WORKSPACE_ENTER_FIXTURES: list[WorkspaceEnterFixture] = [
+    WorkspaceEnterFixture(
+        test_id="pane_enter_false_shortform",
+        yaml=textwrap.dedent(
+            """
 session_name: Should not execute
 windows:
 - panes:
   - shell_command: echo "___$((1 + 3))___"
     enter: false
     """,
-            ),
-            "___4___",
-            False,
-        ],
-        [
-            textwrap.dedent(
-                """
+        ),
+        output="___4___",
+        should_see=False,
+    ),
+    WorkspaceEnterFixture(
+        test_id="pane_enter_false_longform",
+        yaml=textwrap.dedent(
+            """
 session_name: Should not execute
 windows:
 - panes:
@@ -1122,38 +1134,41 @@ windows:
     - echo "___$((1 + 3))___"
     enter: false
     """,
-            ),
-            "___4___",
-            False,
-        ],
-        [
-            textwrap.dedent(
-                """
+        ),
+        output="___4___",
+        should_see=False,
+    ),
+    WorkspaceEnterFixture(
+        test_id="pane_enter_default_shortform",
+        yaml=textwrap.dedent(
+            """
 session_name: Should execute
 windows:
 - panes:
   - shell_command: echo "___$((1 + 3))___"
   """,
-            ),
-            "___4___",
-            True,
-        ],
-        [
-            textwrap.dedent(
-                """
+        ),
+        output="___4___",
+        should_see=True,
+    ),
+    WorkspaceEnterFixture(
+        test_id="pane_enter_default_longform",
+        yaml=textwrap.dedent(
+            """
 session_name: Should execute
 windows:
 - panes:
   - shell_command:
     - echo "___$((1 + 3))___"
   """,
-            ),
-            "___4___",
-            True,
-        ],
-        [
-            textwrap.dedent(
-                """
+        ),
+        output="___4___",
+        should_see=True,
+    ),
+    WorkspaceEnterFixture(
+        test_id="pane_command_enter_false_shortform",
+        yaml=textwrap.dedent(
+            """
 session_name: Should not execute
 windows:
 - panes:
@@ -1161,13 +1176,14 @@ windows:
     - cmd: echo "___$((1 + 3))___"
       enter: false
     """,
-            ),
-            "___4___",
-            False,
-        ],
-        [
-            textwrap.dedent(
-                """
+        ),
+        output="___4___",
+        should_see=False,
+    ),
+    WorkspaceEnterFixture(  # NOQA: PT014 RUF100
+        test_id="pane_command_enter_false_longform",
+        yaml=textwrap.dedent(
+            """
 session_name: Should not execute
 windows:
 - panes:
@@ -1175,25 +1191,27 @@ windows:
     - cmd: echo "___$((1 + 3))___"
       enter: false
     """,
-            ),
-            "___4___",
-            False,
-        ],
-        [
-            textwrap.dedent(
-                """
+        ),
+        output="___4___",
+        should_see=False,
+    ),
+    WorkspaceEnterFixture(  # NOQA: PT014 RUF100
+        test_id="pane_command_enter_default_shortform",
+        yaml=textwrap.dedent(
+            """
 session_name: Should execute
 windows:
 - panes:
   - shell_command: echo "___$((1 + 3))___"
   """,
-            ),
-            "___4___",
-            True,
-        ],
-        [
-            textwrap.dedent(
-                """
+        ),
+        output="___4___",
+        should_see=True,
+    ),
+    WorkspaceEnterFixture(
+        test_id="pane_command_enter_default_longform",
+        yaml=textwrap.dedent(
+            """
 session_name: Should execute
 windows:
 - panes:
@@ -1201,26 +1219,23 @@ windows:
     - cmd: echo "other command"
     - cmd: echo "___$((1 + 3))___"
   """,
-            ),
-            "___4___",
-            True,
-        ],
-    ],
-    ids=[
-        "pane_enter_false_shortform",
-        "pane_enter_false_longform",
-        "pane_enter_default_shortform",
-        "pane_enter_default_longform",
-        "pane_command_enter_false_shortform",
-        "pane_command_enter_false_longform",
-        "pane_command_enter_default_shortform",
-        "pane_command_enter_default_longform",
-    ],
+        ),
+        output="___4___",
+        should_see=True,
+    ),
+]
+
+
+@pytest.mark.parametrize(
+    list(WorkspaceEnterFixture._fields),
+    WORKSPACE_ENTER_FIXTURES,
+    ids=[test.test_id for test in WORKSPACE_ENTER_FIXTURES],
 )
 def test_load_workspace_enter(
     tmp_path: pathlib.Path,
     server: Server,
     monkeypatch: pytest.MonkeyPatch,
+    test_id: str,
     yaml: str,
     output: str,
     should_see: bool,
@@ -1244,21 +1259,28 @@ def test_load_workspace_enter(
 
         if should_see:
             return output in captured_pane
-        else:
-            return output not in captured_pane
+        return output not in captured_pane
 
     assert retry_until(
         fn,
         1,
-    ), f'Should{" " if should_see else "not "} output in captured pane'
+    ), f"Should{' ' if should_see else 'not '} output in captured pane"
 
 
-@pytest.mark.parametrize(
-    "yaml,sleep,output",
-    [
-        [
-            textwrap.dedent(
-                """
+class WorkspaceSleepFixture(t.NamedTuple):
+    """Test fixture for workspace sleep behavior verification."""
+
+    test_id: str
+    yaml: str
+    sleep: float
+    output: str
+
+
+WORKSPACE_SLEEP_FIXTURES: list[WorkspaceSleepFixture] = [
+    WorkspaceSleepFixture(
+        test_id="command_level_sleep_shortform",
+        yaml=textwrap.dedent(
+            """
 session_name: Should not execute
 windows:
 - panes:
@@ -1268,13 +1290,14 @@ windows:
     - cmd: echo "___$((1 + 3))___"
       sleep_before: .35
     """,
-            ),
-            0.5,
-            "___4___",
-        ],
-        [
-            textwrap.dedent(
-                """
+        ),
+        sleep=0.5,
+        output="___4___",
+    ),
+    WorkspaceSleepFixture(
+        test_id="command_level_pane_sleep_longform",
+        yaml=textwrap.dedent(
+            """
 session_name: Should not execute
 windows:
 - panes:
@@ -1284,13 +1307,14 @@ windows:
     - cmd: echo "___$((1 + 3))___"
       sleep_before: .25
     """,
-            ),
-            1.25,
-            "___4___",
-        ],
-        [
-            textwrap.dedent(
-                """
+        ),
+        sleep=1.25,
+        output="___4___",
+    ),
+    WorkspaceSleepFixture(
+        test_id="pane_sleep_shortform",
+        yaml=textwrap.dedent(
+            """
 session_name: Should not execute
 windows:
 - panes:
@@ -1298,13 +1322,14 @@ windows:
     - cmd: echo "___$((1 + 3))___"
     sleep_before: .5
     """,
-            ),
-            0.5,
-            "___4___",
-        ],
-        [
-            textwrap.dedent(
-                """
+        ),
+        sleep=0.5,
+        output="___4___",
+    ),
+    WorkspaceSleepFixture(
+        test_id="pane_sleep_longform",
+        yaml=textwrap.dedent(
+            """
 session_name: Should not execute
 windows:
 - panes:
@@ -1312,13 +1337,14 @@ windows:
     - cmd: echo "___$((1 + 3))___"
     sleep_before: 1
     """,
-            ),
-            1,
-            "___4___",
-        ],
-        [
-            textwrap.dedent(
-                """
+        ),
+        sleep=1,
+        output="___4___",
+    ),
+    WorkspaceSleepFixture(
+        test_id="shell_before_before_command_level",
+        yaml=textwrap.dedent(
+            """
 session_name: Should not execute
 shell_command_before:
   - cmd: echo "sleeping before"
@@ -1327,26 +1353,26 @@ windows:
 - panes:
   - echo "___$((1 + 3))___"
     """,
-            ),
-            0.5,
-            "___4___",
-        ],
-    ],
-    ids=[
-        "command_level_sleep_shortform",
-        "command_level_pane_sleep_longform",
-        "pane_sleep_shortform",
-        "pane_sleep_longform",
-        "shell_before_before_command_level",
-    ],
+        ),
+        sleep=0.5,
+        output="___4___",
+    ),
+]
+
+
+@pytest.mark.parametrize(
+    list(WorkspaceSleepFixture._fields),
+    WORKSPACE_SLEEP_FIXTURES,
+    ids=[test.test_id for test in WORKSPACE_SLEEP_FIXTURES],
 )
 @pytest.mark.flaky(reruns=3)
 def test_load_workspace_sleep(
     tmp_path: pathlib.Path,
     server: Server,
     monkeypatch: pytest.MonkeyPatch,
+    test_id: str,
     yaml: str,
-    sleep: int,
+    sleep: float,
     output: str,
 ) -> None:
     """Test sleep commands in tmuxp configuration."""
@@ -1406,10 +1432,10 @@ def test_first_pane_start_directory(session: Session, tmp_path: pathlib.Path) ->
             pane_path = p.pane_current_path
             return (pane_path is not None and path in pane_path) or pane_path == path
 
-        _f = functools.partial(f, path=path, p=p)
+        f_ = functools.partial(f, path=path, p=p)
 
         # handle case with OS X adding /private/ to /tmp/ paths
-        assert retry_until(_f)
+        assert retry_until(f_)
 
 
 @pytest.mark.skipif(
@@ -1442,9 +1468,9 @@ def test_layout_main_horizontal(session: Session) -> None:
     assert all(
         main_horizontal_pane_height != pane_height for pane_height in pane_heights
     ), "The top row should not be the same size as the bottom row (even though it can)"
-    assert all(
-        pane_heights[0] == pane_height for pane_height in pane_heights
-    ), "The bottom row should be uniform height"
+    assert all(pane_heights[0] == pane_height for pane_height in pane_heights), (
+        "The bottom row should be uniform height"
+    )
     assert width(main_horizontal_pane) > width(panes[0])
 
     def is_almost_equal(x: int, y: int) -> bool:
@@ -1461,9 +1487,9 @@ class DefaultSizeNamespaceFixture(t.NamedTuple):
     test_id: str
 
     # test params
-    TMUXP_DEFAULT_SIZE: t.Optional[str]
+    TMUXP_DEFAULT_SIZE: str | None
     raises: bool
-    confoverrides: t.Dict[str, t.Any]
+    confoverrides: dict[str, t.Any]
 
 
 DEFAULT_SIZE_FIXTURES = [
@@ -1495,12 +1521,12 @@ DEFAULT_SIZE_FIXTURES = [
 )
 @pytest.mark.skipif(has_lt_version("2.9"), reason="default-size only applies there")
 def test_issue_800_default_size_many_windows(
-    server: "Server",
+    server: Server,
     monkeypatch: pytest.MonkeyPatch,
     test_id: str,
-    TMUXP_DEFAULT_SIZE: t.Optional[str],
+    TMUXP_DEFAULT_SIZE: str | None,
     raises: bool,
-    confoverrides: t.Dict[str, t.Any],
+    confoverrides: dict[str, t.Any],
 ) -> None:
     """Recreate default-size issue.
 
@@ -1508,7 +1534,12 @@ def test_issue_800_default_size_many_windows(
     a lot of panes.
 
     See also: https://github.com/tmux-python/tmuxp/issues/800
+
+    2024-04-07: This test isn't being used as of this date, as default-size is totally
+    unused in builder.py.
     """
+    monkeypatch.setenv("ROWS", "36")
+
     yaml_workspace = test_utils.get_workspace_file(
         "regressions/issue_800_default_size_many_windows.yaml",
     )
@@ -1521,9 +1552,6 @@ def test_issue_800_default_size_many_windows(
         for k, v in confoverrides.items():
             workspace[k] = v
 
-    if TMUXP_DEFAULT_SIZE is not None:
-        monkeypatch.setenv("TMUXP_DEFAULT_SIZE", TMUXP_DEFAULT_SIZE)
-
     builder = WorkspaceBuilder(session_config=workspace, server=server)
 
     if raises:
